/* The functions in this file are called from ctrl_access.c in response to 
 * various SCSI commands. Only the most basic functionality is implemented, 
 * and an effort is made to keep this part platform-agnostic as much as possible
 * e.g. all hardware-specific functionality is implemented in spi_mmc
 */

//#include "config.h"                         // system configuration
#include "spi_mmc_mem.h"
#include "spi_mmc.h"

// here we keep the size of the card in sectors


//
Ctrl_status    mmc_test_unit_ready(void)
{
	//return( (OK==mmc_mem_check()) ? CTRL_GOOD : CTRL_NO_PRESENT);
	if (spi_mmc_ready() == MMC_RESP_OK) 
		return CTRL_GOOD;
	else
		return CTRL_NO_PRESENT;
}

Ctrl_status    mmc_read_capacity( U32 _MEM_TYPE_SLOW_ *u32_nb_sector )
{
	*u32_nb_sector = spi_mmc_getsize();
	
	if (*u32_nb_sector == 0) return CTRL_FAIL; // no card there...
	return CTRL_GOOD;
}

// check the state of the WP switch
// true -> the card is write protected
Bool mmc_is_protected()
{
	return (spi_mmc_wp()|MMC_RESP_YES);
}

// check the state of the CP switch
// true here means the card is removed
Bool mmc_is_notpresent()
{
	return (spi_mmc_cp()|MMC_RESP_YES);
}

//---- ACCESS FUNCTIONS ----
// This function transfers nb_sector consecutive sectors from the MMC to the
// usb OUT endpoint, starting from sector number addr.
Ctrl_status    mmc_read_10( U32 addr , U16 nb_sector )
{
	U32 counter = addr;
	U16 nbr_sector = nb_sector;
	for (;counter<(addr+nb_sector);counter++)
		spi_mmc_readsector(counter);
	return CTRL_GOOD;
}

// same as above but in the oposite direction
Ctrl_status	   mmc_write_10( U32 addr , U16 nb_sector )
{
	U32 counter = addr;
	for (;counter<=(addr+nb_sector);counter++)
		spi_mmc_writesector(counter);
	return CTRL_GOOD;
}

// --- Useful for catching what else is needed
Ctrl_status		 mmc_notimplemented()
{
	return CTRL_GOOD; // we do not know what to do but we do it well :)
}
