/****************************************************************************
 *  
 * Project: Olimex AVR ISP with MKII protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: massbulk.h
 * Description: USB Mass Storage Bulk-Only Module
 * Developer: Dimitar Dimitrov ( dinuxbg __on__ gmail.com )
 *
 * Last change: $Date: 2007-11-28 17:54:56 +0200 (сряда, 28 Ноември 2007) $
 * Revision: $Revision: 64 $
 * Id: $Id: massbulk.h 64 2007-11-28 15:54:56Z dimitar $
 *
 ****************************************************************************/

#ifndef MASSBULK_H
#define MASSBULK_H

/** Control Class-Specific Requests. */
#define MS_CLSREQ_MASS_STORAGE_RESET			0xFF
#define MS_CLSREQ_GET_MAX_LUN				0xFE

/** USB Mass Storage interface class codes. */
#define USB_INTRFCLASS_MASS_STORAGE			0x08
#define USB_INTRFSUBCLASS_REDUCED_BLOCK_COMMANDS	0x01
#define USB_INTRFSUBCLASS_SFF8020_MMC2			0x02
#define USB_INTRFSUBCLASS_QIC157			0x03
#define USB_INTRFSUBCLASS_UFI				0x04
#define USB_INTRFSUBCLASS_SFF8070			0x05
#define USB_INTRFSUBCLASS_SCSI_TRANSPARENT		0x06

/** USB Mass Storage interface protocol codes. */
#define USB_INTRFPROT_CBI_CMDCOMPLINT			0x00
#define USB_INTRFPROT_CBI_NOCMDCOMPLINT			0x01
#define USB_INTRFPROT_BULK_ONLY				0x50


/** Control Block Wrapper expected signature. */
#define MS_CBW_SIGNATURE				0x43425355UL

/** Control Status Wrapper expected signature. */
#define MS_CSW_SIGNATURE				0x53425355UL

/** Command Status status values. */
enum ms_csw_status {
	MS_CSW_STATUS_COMMAND_PASSED = 0x00,
	MS_CSW_STATUS_COMMAND_FAILED = 0x01,
	MS_CSW_STATUS_PHASE_ERROR = 0x02
};

/** For now we support only one Logical Units. If more are needed
 * then a change of the driver would be needed! */
#define MS_MAX_LUN					0

/** USB Mass Storage Bulk-Only Command Block Wrapper. */
struct ms_cbw {
	uint32_t signature;		/**< Signature that helps itentify the
					 * packet as a CBW. Shall contain the 
					 * value MS_CBW_SIGNATURE. */
	uint32_t tag;			/**< Tag that we must echo back. */
	uint32_t data_transfer_length;	/**< Number of bytes of data that the
					 * host expects to transfer on the
					 * Bulk-In/Out endpoint. */
	uint8_t reserved_a : 6;
	uint8_t reserved_b : 1;
	uint8_t dir_d2h : 1;	/**< Device shall ignore this bit if
				 * data length is zero, otherwise:
				 *    0 = Data-Out from host to device
				 *    1 = Data-In from device to host
				 */
	uint8_t lun;			/**< Logical Unit Number to which
					 * the command block is sent. */
	uint8_t cb_length;		/**< The valid length of the cb[] 
					 * field in bytes. */
	uint8_t cb[16];			/**< The command block to be executed
					 * by the device.
					 */
};

/** USB Mass Storage Bulk-Only Command Status Wrapper. */
struct ms_csw {
	uint32_t signature;		/**< Signature that helps identify
					 * this packet as a CSW. */
	uint32_t tag;			/**< Shall be what was sent with CBW */
	uint32_t data_residue;		/**< The device shall report the 
					 * difference between the amount of 
					 * data expected and the actual amount 
					 * of data processed/sent by the 
					 * device.
					 */
	enum ms_csw_status status;	/**< Indicates the success or failure
					 * of the command. */
};

extern void ms_poll(void);

#endif /* MASSBULK_H */

